@@ -13,10 +13,11 @@
 # See the License for the specific language governing permissions and        #
 # limitations under the License.                                             #
 # -------------------------------------------------------------------------- #
+import json
 
 """
 Commands that directly invoke the API. Most of these commands are a one-to-one
-mapping to the API, although some some (like gp-add-host) simply provide 
+mapping to the API, although some some (like gp-instance-add-host) simply provide 
 a more convenient interface on top of instance_update().
 """
 
@@ -36,15 +37,15 @@ from globus.provision.core.config import SimpleTopologyConfig
 from globus.provision.common.config import ConfigException
 
 
-def gp_create_func():
-    return gp_create(sys.argv).run()
+def gp_instance_create_func():
+    return gp_instance_create(sys.argv).run()
         
-class gp_create(Command):
+class gp_instance_create(Command):
     """
     Creates a new Globus Provision instance.
     """
     
-    name = "gp-create"
+    name = "gp-instance-create"
 
     def __init__(self, argv):
         Command.__init__(self, argv)
@@ -103,12 +104,13 @@ class gp_create(Command):
         else:
             print "Created new instance:",
             print Fore.WHITE + Style.BRIGHT + inst_id
+            self._set_last_gpi(inst_id)
             
             
-def gp_describe_instance_func():
-    return gp_describe_instance(sys.argv).run()            
+def gp_instance_describe_func():
+    return gp_instance_describe(sys.argv).run()            
         
-class gp_describe_instance(Command):
+class gp_instance_describe(Command):
     """
     Describes a Globus Provision instance, providing information on the state of the instance,
     and of the individual hosts (including their hostnames and IPs, if the instance is running). 
@@ -117,52 +119,14 @@ class gp_describe_instance(Command):
     
     The instance identifier must be specified after all other parameters. For example::
     
-        gp_describe_instance --verbose gpi-12345678
+        gp-instance-describe --verbose gpi-12345678
         
     """
     
-    name = "gp-describe-instance"
+    name = "gp-instance-describe"
     
     def __init__(self, argv):
         Command.__init__(self, argv)
-        
-    def __colorize_topology_state(self, state):
-        state_str = Topology.state_str[state]
-        reset = Fore.RESET + Style.RESET_ALL
-        if state == Topology.STATE_NEW:
-            return Fore.BLUE + Style.BRIGHT + state_str + reset
-        elif state == Topology.STATE_RUNNING:
-            return Fore.GREEN + Style.BRIGHT + state_str + reset
-        elif state in (Topology.STATE_STARTING, Topology.STATE_CONFIGURING, Topology.STATE_STOPPING, Topology.STATE_RESUMING, Topology.STATE_TERMINATING):
-            return Fore.YELLOW + Style.BRIGHT + state_str + reset
-        elif state in (Topology.STATE_TERMINATED, Topology.STATE_FAILED):
-            return Fore.RED + Style.BRIGHT + state_str + reset
-        elif state == Topology.STATE_STOPPED:
-            return Fore.MAGENTA + Style.BRIGHT + state_str + reset
-        else:
-            return state_str
-
-    def __colorize_node_state(self, state):
-        state_str = Node.state_str[state]
-        reset = Fore.RESET + Style.RESET_ALL
-        if state == Node.STATE_NEW:
-            return Fore.BLUE + Style.BRIGHT + state_str + reset
-        elif state == Node.STATE_RUNNING:
-            return Fore.GREEN + Style.BRIGHT + state_str + reset
-        elif state in (Node.STATE_STARTING, Node.STATE_RUNNING_UNCONFIGURED, Node.STATE_CONFIGURING, Node.STATE_STOPPING, Node.STATE_RESUMING, Node.STATE_TERMINATING):
-            return Fore.YELLOW + Style.BRIGHT + state_str + reset
-        elif state in (Node.STATE_TERMINATED, Node.STATE_FAILED):
-            return Fore.RED + Style.BRIGHT + state_str + reset
-        elif state == Node.STATE_STOPPED:
-            return Fore.MAGENTA + Style.BRIGHT + state_str + reset
-        else:
-            return state_str
-        
-    def __pad(self, str, colorstr, width):
-        if colorstr == "":
-            return str.ljust(width)
-        else:
-            return colorstr + " " * (width - len(str))
                 
     def run(self):    
         self.parse_options()
@@ -181,12 +145,14 @@ class gp_describe_instance(Command):
             self._print_error("Could not access instance.", message)
             exit(1) 
         else:
+            self._set_last_gpi(inst_id)
+            
             if self.opt.verbose or self.opt.debug:
                 print topology_json
             else:
                 topology = Topology.from_json_string(topology_json)
                 reset = Fore.RESET + Style.RESET_ALL
-                print Fore.WHITE + Style.BRIGHT + inst_id + reset + ": " + self.__colorize_topology_state(topology.state)
+                print Fore.WHITE + Style.BRIGHT + inst_id + reset + ": " + self._colorize_topology_state(topology.state)
                 print
                 for domain in topology.domains.values():
                     print "Domain " + Fore.CYAN + "'%s'" % domain.id
@@ -226,30 +192,30 @@ class gp_describe_instance(Command):
                         rows.append((node.id, state, state_str, hostname, ip))                          
                                                 
                     for (node_id, state, state_str, hostname, ip) in rows:
-                        node_id_pad = self.__pad(node_id, Fore.WHITE + Style.BRIGHT + node_id + Fore.RESET + Style.RESET_ALL, node_width + 2) 
-                        state_pad = self.__pad(state_str, self.__colorize_node_state(state), state_width + 2) 
-                        hostname_pad   = self.__pad(hostname, "", hostname_width + 2)
-                        ip_pad = self.__pad(ip, "", ip_width)
+                        node_id_pad = self._pad(node_id, Fore.WHITE + Style.BRIGHT + node_id + Fore.RESET + Style.RESET_ALL, node_width + 2) 
+                        state_pad = self._pad(state_str, self._colorize_node_state(state), state_width + 2) 
+                        hostname_pad   = self._pad(hostname, "", hostname_width + 2)
+                        ip_pad = self._pad(ip, "", ip_width)
                         print "    " + node_id_pad + state_pad + hostname_pad + ip_pad
                     print
                     
-def gp_start_func():
-    return gp_start(sys.argv).run()  
+def gp_instance_start_func():
+    return gp_instance_start(sys.argv).run()  
 
-class gp_start(Command):
+class gp_instance_start(Command):
     """
-    Starts a Globus Provision instance. If the instance was previous stopped, ``gp-start``
+    Starts a Globus Provision instance. If the instance was previous stopped, ``gp-instance-start``
     will resume it.
     
     See :ref:`sec_test_chef` for details on how to use the ``--extra-files`` option.
     
     The instance identifier must be specified after all other parameters. For example::
     
-        gp-start --extra-files foo.txt gpi-12345678
+        gp-instance-start --extra-files foo.txt gpi-12345678
         
     """
         
-    name = "gp-start"
+    name = "gp-instance-start"
     
     def __init__(self, argv):
         Command.__init__(self, argv)
@@ -293,6 +259,9 @@ class gp_start(Command):
         
         if status_code == API.STATUS_SUCCESS:
             print Fore.GREEN + Style.BRIGHT + "done!"
+
+            self._set_last_gpi(inst_id)
+            
             t_end = time.time()
             
             delta = t_end - t_start
@@ -304,10 +273,10 @@ class gp_start(Command):
             self._print_error("Could not start instance.", message)
             exit(1) 
 
-def gp_update_topology_func():
-    return gp_update_topology(sys.argv).run()  
+def gp_instance_update_func():
+    return gp_instance_update(sys.argv).run()  
 
-class gp_update_topology(Command):
+class gp_instance_update(Command):
     """
     Updates a Globus Provision instance's topology. Globus Provision will determine what changes
     and necessary (adding/removing hosts, etc.) and will return an error if an invalid update
@@ -317,11 +286,11 @@ class gp_update_topology(Command):
     
     The instance identifier must be specified after all other parameters. For example::
     
-        gp-update-topology --topology newtopology.json gpi-12345678
+        gp-instance-update --topology newtopology.json gpi-12345678
         
     """
         
-    name = "gp-update-topology"
+    name = "gp-instance-update"
     
     def __init__(self, argv):
         Command.__init__(self, argv)
@@ -380,6 +349,9 @@ class gp_update_topology(Command):
         
         if status_code == API.STATUS_SUCCESS:
             print Fore.GREEN + Style.BRIGHT + "done!"
+            
+            self._set_last_gpi(inst_id)
+            
             t_end = time.time()
             
             delta = t_end - t_start
@@ -391,23 +363,23 @@ class gp_update_topology(Command):
             exit(1)
 
 
-def gp_stop_func():
-    return gp_stop(sys.argv).run()         
+def gp_instance_stop_func():
+    return gp_instance_stop(sys.argv).run()         
         
-class gp_stop(Command):
+class gp_instance_stop(Command):
     """
     Stops a running Globus Provision instance. This will shut down all the hosts in the instance,
-    but it will not free the corresponding resources. You can use :ref:`cli_gp-start` to resume
-    the instance at a later time. Use :ref:`cli_gp-terminate` if you want to shut down the hosts
+    but it will not free the corresponding resources. You can use :ref:`cli_gp-instance-start` to resume
+    the instance at a later time. Use :ref:`cli_gp-instance-terminate` if you want to shut down the hosts
     *and* free all their resources (including all associated storage) 
     
     The instance identifier must be specified after all other parameters. For example::
     
-        gp-stop --verbose gpi-12345678
+        gp-instance-stop --verbose gpi-12345678
         
     """
         
-    name = "gp-stop"
+    name = "gp-instance-stop"
     
     def __init__(self, argv):
         Command.__init__(self, argv)
@@ -430,33 +402,34 @@ class gp_stop(Command):
         
         if status_code == API.STATUS_SUCCESS:
             print Fore.GREEN + Style.BRIGHT + "done!"
+            self._set_last_gpi(inst_id)
         elif status_code == API.STATUS_FAIL:
             self._print_error("Could not stop instance.", message)
             print
             exit(1)       
 
 
-def gp_terminate_func():
-    return gp_terminate(sys.argv).run()     
+def gp_instance_terminate_func():
+    return gp_instance_terminate(sys.argv).run()     
 
-class gp_terminate(Command):
+class gp_instance_terminate(Command):
     """
     Terminates a Globus Provision instance. This not only shuts down all the hosts in the
     instance, but also frees up all associated resources, including storage. Use this command
     only if you want to irreversibly "kill" your instance. If you only want to stop it temporarily
     (shutting down the hosts, but allowing them to be resumed at a later time), use 
-    :ref:`cli_gp-stop` instead.
+    :ref:`cli_gp-instance-stop` instead.
     
     This command can also be used on instances that are in the "Failed" state, to free their
     resources.
     
     The instance identifier must be specified after all other parameters. For example::
     
-        gp-terminate --verbose gpi-12345678
+        gp-instance-terminate --verbose gpi-12345678
         
     """
     
-    name = "gp-terminate"
+    name = "gp-instance-terminate"
     
     def __init__(self, argv):
         Command.__init__(self, argv)
@@ -479,27 +452,28 @@ class gp_terminate(Command):
         
         if status_code == API.STATUS_SUCCESS:
             print Fore.GREEN + Style.BRIGHT + "done!"
+            self._set_last_gpi(inst_id)
         elif status_code == API.STATUS_FAIL:
             print
             self._print_error("Could not terminate instance.", message)
             exit(1)  
 
 
-def gp_list_instances_func():
-    return gp_list_instances(sys.argv).run()     
+def gp_instance_list_func():
+    return gp_instance_list(sys.argv).run()     
 
-class gp_list_instances(Command):
+class gp_instance_list(Command):
     """
     Lists all instances you've created, showing their identifier and their state.
     
     If you only want to list certain instances, you can specify a list of instance
     identifiers after all other parameters. For example::
     
-        gp-list-instances --verbose gpi-11111111 gpi-22222222 gpi-33333333
+        gp-instance-list --verbose gpi-11111111 gpi-22222222 gpi-33333333
         
     """
         
-    name = "gp-list-instances"
+    name = "gp-instance-list"
     
     def __init__(self, argv):
         Command.__init__(self, argv)
@@ -513,32 +487,39 @@ class gp_list_instances(Command):
             inst_ids = None
         
         api = API(self.opt.dir)
-        insts = api.instance_list(inst_ids)
+        (status_code, message, topologies_json) = api.instance_list(inst_ids)
         
-        for i in insts:
-            t = i.topology
-            print "%s\t%s" % (i.id, Topology.state_str[t.state])
-            if self.opt.verbose:
-                for node in t.get_nodes():
-                    print "\t%s\t%s\t%s" % (node.id, node.hostname, node.ip)
-                    if self.opt.debug:
-                        print "\t%s" % node.deploy_data
+        
+        if status_code != API.STATUS_SUCCESS:
+            self._print_error("Unable to list instances.", message)
+            exit(1) 
+        else:        
+            insts = json.loads(topologies_json)
+            
+            for inst in insts:
+                if self.opt.verbose or self.opt.debug:
+                    print json.dumps(inst, indent=2)
+                else:
+                    topology = Topology.from_json_dict(inst)
+                    
+                    reset = Fore.RESET + Style.RESET_ALL
+                    print Fore.WHITE + Style.BRIGHT + topology.id + reset + ": " + self._colorize_topology_state(topology.state)
 
 
-def gp_add_user_func():
-    return gp_add_user(sys.argv).run()     
+def gp_instance_add_user_func():
+    return gp_instance_add_user(sys.argv).run()     
 
-class gp_add_user(Command):
+class gp_instance_add_user(Command):
     """
     Adds a user to a running instance.
     
     The instance identifier must be specified after all other parameters. For example::
     
-        gp-add-user --domain simple --login johnsmith gpi-12345678
+        gp-instance-add-user --domain simple --login johnsmith gpi-12345678
         
     """
         
-    name = "gp-add-user"
+    name = "gp-instance-add-user"
     
     def __init__(self, argv):
         Command.__init__(self, argv)    
@@ -618,6 +599,9 @@ class gp_add_user(Command):
             
             if status_code == API.STATUS_SUCCESS:
                 print Fore.GREEN + Style.BRIGHT + "done!"
+
+                self._set_last_gpi(inst_id)
+                
                 t_end = time.time()
                 
                 delta = t_end - t_start
@@ -629,20 +613,20 @@ class gp_add_user(Command):
                 exit(1)
         
         
-def gp_add_host_func():
-    return gp_add_host(sys.argv).run()     
+def gp_instance_add_host_func():
+    return gp_instance_add_host(sys.argv).run()     
         
-class gp_add_host(Command):
+class gp_instance_add_host(Command):
     """
     Adds a new host to a running instance.
     
     The instance identifier must be specified after all other parameters. For example::
     
-        gp-add-host --domain simple --id simple-gridftp --run-list role[domain-gridftp] gpi-12345678
+        gp-instance-add-host --domain simple --id simple-gridftp --run-list role[domain-gridftp-default] gpi-12345678
         
     """
         
-    name = "gp-add-host"
+    name = "gp-instance-add-host"
     
     def __init__(self, argv):
         Command.__init__(self, argv)
@@ -707,6 +691,9 @@ class gp_add_host(Command):
             
             if status_code == API.STATUS_SUCCESS:
                 print Fore.GREEN + Style.BRIGHT + "done!"
+                
+                self._set_last_gpi(inst_id)
+                
                 t_end = time.time()
                 
                 delta = t_end - t_start
@@ -718,21 +705,21 @@ class gp_add_host(Command):
                 exit(1) 
         
         
-def gp_remove_users_func():
-    return gp_remove_users(sys.argv).run()     
+def gp_instance_remove_users_func():
+    return gp_instance_remove_users(sys.argv).run()     
 
-class gp_remove_users(Command):
+class gp_instance_remove_users(Command):
     """
     Removes users from a running instance.
     
     The logins of the users to be removed must be specified after the instance identifier which,
     in turn, must be specified after all other parameters. For example::
     
-        gp-remove-users --domain simple gpi-12345678 johnsmith sarahjane
+        gp-instance-remove-users --domain simple gpi-12345678 johnsmith sarahjane
         
     """
         
-    name = "gp-remove-users"
+    name = "gp-instance-remove-users"
     
     def __init__(self, argv):
         Command.__init__(self, argv)
@@ -789,6 +776,9 @@ class gp_remove_users(Command):
     
                 if status_code == API.STATUS_SUCCESS:
                     print Fore.GREEN + Style.BRIGHT + "done!"
+                    
+                    self._set_last_gpi(inst_id)
+                    
                     t_end = time.time()
                     
                     delta = t_end - t_start
@@ -801,21 +791,21 @@ class gp_remove_users(Command):
 
         
         
-def gp_remove_hosts_func():
-    return gp_remove_hosts(sys.argv).run()             
+def gp_instance_remove_hosts_func():
+    return gp_instance_remove_hosts(sys.argv).run()             
         
-class gp_remove_hosts(Command):
+class gp_instance_remove_hosts(Command):
     """
     Removes hosts from a running instance.
     
     The host identifiers must be specified after the instance identifier which,
     in turn, must be specified after all other parameters. For example::
     
-        gp-remove-hosts --domain simple gpi-12345678 simple-gridftp simple-condor
+        gp-instance-remove-hosts --domain simple gpi-12345678 simple-gridftp simple-condor
         
     """
         
-    name = "gp-remove-hosts"
+    name = "gp-instance-remove-hosts"
     
     def __init__(self, argv):
         Command.__init__(self, argv)
@@ -872,6 +862,9 @@ class gp_remove_hosts(Command):
     
                 if status_code == API.STATUS_SUCCESS:
                     print Fore.GREEN + Style.BRIGHT + "done!"
+                    
+                    self._set_last_gpi(inst_id)
+                    
                     t_end = time.time()
                     
                     delta = t_end - t_start
