@@ -13,7 +13,6 @@
 # See the License for the specific language governing permissions and        #
 # limitations under the License.                                             #
 # -------------------------------------------------------------------------- #
-
 """
 The Globus Provision API
 
@@ -26,6 +25,7 @@ API that supports multiple users.
 
 import sys
 import traceback
+import os.path
 
 from boto.exception import EC2ResponseError
 
@@ -40,6 +40,8 @@ from globus.provision.common.ssh import SSHCommandFailureException
 from globus.provision.common import log
 from globus.provision.common.config import ConfigException
 from globus.provision.common.persistence import ObjectValidationException
+from globus.provision.common.go_transfer import GlobusOnlineCLIHelper, GlobusOnlineHelper,\
+    GlobusOnlineException
 
 class API(object):
     """
@@ -115,6 +117,13 @@ class API(object):
                 inst.topology.state = Topology.STATE_RESUMING
     
             inst.topology.save()
+
+            try:
+                self.__globusonline_pre_start(inst)
+            except GlobusOnlineException, goe:
+                log.warning("Unable to create GO endpoint/s: %s" % goe)
+               
+            inst.topology.save()                            
             
             nodes = inst.topology.get_nodes()
     
@@ -134,7 +143,7 @@ class API(object):
                 deployer.post_allocate(node, vm)
 
             inst.topology.save()
-    
+                            
             # Generate certificates
             if not resuming:
                 inst.gen_certificates(force_hosts=False, force_users=False)
@@ -153,7 +162,13 @@ class API(object):
                 return (API.STATUS_FAIL, message)
     
             inst.topology.state = Topology.STATE_RUNNING
-            inst.topology.save()
+            inst.topology.save()         
+            
+            log.info("Creating Globus Online endpoints")
+            try:
+                self.__globusonline_post_start(inst)            
+            except GlobusOnlineException, goe:
+                log.warning("Unable to create GO endpoint/s: %s" % goe)
             
             return (API.STATUS_SUCCESS, "Success")
         except:
@@ -176,7 +191,19 @@ class API(object):
     
             log.set_logging_instance(inst_id)
                             
-            if inst.topology.state != Topology.STATE_RUNNING:
+            if inst.topology.state == Topology.STATE_NEW:
+                # If the topology is still in a New state, we simply
+                # validate that the update is valid, and replace
+                # the old topology. We don't need to deploy or
+                # configure any hosts..
+                if topology_json != None:
+                    try:
+                        (success, message, create_hosts, destroy_hosts) = inst.update_topology(topology_json)
+                    except ObjectValidationException, ove:
+                        message = "Error in topology file: %s" % ove
+                        return (API.STATUS_FAIL, message)
+                return (API.STATUS_SUCCESS, "Success")
+            elif inst.topology.state != Topology.STATE_RUNNING:
                 message = "Cannot update the topology of an instance that is in state '%s'" % (Topology.state_str[inst.topology.state])
                 return (API.STATUS_FAIL, message)        
     
@@ -348,10 +375,26 @@ class API(object):
                 inst.topology.save()
                 return (API.STATUS_FAIL, message)            
         
+            go_helper = GlobusOnlineHelper.from_instance(inst)
+            
+            try:
+                # Remove GO endpoints
+                for domain_name, domain in inst.topology.domains.items():
+                    if domain.has_property("go_endpoints"):
+                        for ep in domain.go_endpoints:     
+                            go_helper.connect(ep.user)
+                            try:
+                                go_helper.endpoint_remove(ep)
+                            except:
+                                pass   
+                            go_helper.disconnect()
+            except GlobusOnlineException, goe:
+                log.warning("Unable to remove GO endpoint/s: %s" % goe)
+        
             inst.topology.state = Topology.STATE_TERMINATED
             inst.topology.save()
               
-            log.info("Instances have been stopped running.")
+            log.info("Instances have been terminated.")
             return (API.STATUS_SUCCESS, "Success")
         except:
             message = self.__unexpected_exception_to_text(what = "starting the instance.")
@@ -367,13 +410,16 @@ class API(object):
     def instance_list(self, inst_ids):
         istore = InstanceStore(self.instances_dir)
         
-        if inst_ids == None:
-            insts = istore.get_instances()
-        else:
-            insts = [istore.get_instance(inst_id) for inst_id in inst_ids]
+        (valid, invalid) = istore.get_instances(inst_ids)
+        instances_jsons = []
+        for inst in valid:
+            instances_jsons.append(inst.topology.to_json_string())
+            
+        instances_json = "[" + ",".join(instances_jsons) + "]"
                 
-        # TODO: Return JSON
-        return insts
+        # TODO: Return invalid instances as "warnings", once we
+        # add that extra return value to the API.
+        return API.STATUS_SUCCESS, "Success", instances_json
 
         
     def __get_instance(self, inst_id):
@@ -401,6 +447,38 @@ class API(object):
             
         return deploy_module.Deployer        
         
+
+    def __globusonline_pre_start(self, inst):
+        go_helper = GlobusOnlineHelper.from_instance(inst)
+        
+        for domain_name, domain in inst.topology.domains.items():
+            if domain.has_property("go_endpoints"):
+                for ep in domain.go_endpoints:
+                    if ep.has_property("globus_connect_cert") and ep.globus_connect_cert:
+                        if ep.gridftp.startswith("node:"):
+                            gridftp_node = inst.topology.get_node_by_id(ep.gridftp[5:])
+                            go_helper.connect(ep.user)
+                            gc_setupkey = go_helper.endpoint_gc_create(ep, replace = True)
+                            gridftp_node.set_property("gc_setupkey", gc_setupkey)
+                            go_helper.disconnect()        
+
+
+    def __globusonline_post_start(self, inst):
+        go_helper = GlobusOnlineHelper.from_instance(inst)
+        
+        # Globus Online
+        for domain_name, domain in inst.topology.domains.items():
+            if domain.has_property("go_endpoints"):
+                for ep in domain.go_endpoints:
+                    go_helper.connect(ep.user)
+                    if ep.has_property("globus_connect_cert") and ep.globus_connect_cert:
+                        if ep.gridftp.startswith("node:"):
+                            go_helper.endpoint_gc_create_finalize(ep)
+                    else:        
+                        go_helper.create_endpoint(ep, replace=True)
+                    go_helper.disconnect()
+                    
+        
     def __allocate_vms(self, deployer, nodes, resuming):
         # TODO: Make this an option
         sequential = False
