@@ -159,7 +159,7 @@ class GPConfig(Config):
             """),
      Option(name        = "server-port",
             getter      = "ec2-server-port",
-            type        = OPTTYPE_STRING,
+            type        = OPTTYPE_INT,
             required    = False,
             doc         = """
             The TCP port of the EC2 server. If you are using Amazon AWS, leave this option
@@ -194,6 +194,7 @@ class GPConfig(Config):
                   """)
     go.options = \
     [       
+   
      Option(name        = "ssh-key",
             getter      = "go-ssh-key",
             type        = OPTTYPE_FILE,
@@ -370,6 +371,17 @@ class SimpleTopologyConfig(Config):
                 N madduri
             
             """),   
+     
+     Option(name        = "barebones-nodes",
+            getter      = "barebones-nodes",
+            type        = OPTTYPE_INT,
+            default     = 0,
+            required    = False,
+            doc         = """
+            A "barebones node" is a node on which no software will be installed. If ``nfs-nis`` is ``True``,
+            these nodes *will* be configured as NFS/NIS clients. These nodes can be useful for testing.   
+            """),            
+     
      Option(name        = "nfs-nis",
             getter      = "nfs-nis",
             type        = OPTTYPE_BOOLEAN,
@@ -442,7 +454,8 @@ class SimpleTopologyConfig(Config):
             default     = False,
             doc         = """
             Specifies whether to set up a Galaxy server on this domain.        
-            """),            
+            """),   
+              
      Option(name        = "go-endpoint",
             getter      = "go-endpoint",
             type        = OPTTYPE_STRING,
@@ -455,8 +468,8 @@ class SimpleTopologyConfig(Config):
             :ref:`[globusonline] section<GPConfig_section_globusonline>` of the configuration file.
             
             See :ref:`chap_go` for more details.    
-            """)
-,          
+            """),
+               
      Option(name        = "go-auth",
             getter      = "go-auth",
             type        = OPTTYPE_STRING,
@@ -473,6 +486,20 @@ class SimpleTopologyConfig(Config):
               
             See :ref:`chap_go`, and specifically :ref:`Globus Online Authentication Methods <sec_go_auth>`,
             for more details on the implications of each authentication method.            
+            """),
+     
+     Option(name        = "go-gc",
+            getter      = "go-gc",
+            type        = OPTTYPE_BOOLEAN,
+            required    = False,
+            default     = True,
+            doc         = """
+            If true, the endpoint will use a Globus Connect certificate.
+            If not, it will use the host certificate generated by Globus Provision.
+            Take into account that, for the GridFTP server to be trusted by
+            Globus Online, it must use a certificate trusted by Globus Online.
+            Unless you used a CA trusted by Globus Online to generate the certificates
+            for the topology, you must use a Globus Connect certificate.
             """)                            
     ]     
     sections.append(domain)
@@ -519,6 +546,7 @@ class SimpleTopologyConfig(Config):
     ]    
     sections.append(ec2)    
   
+ 
     def __init__(self, configfile):
         Config.__init__(self, configfile, self.sections)
 
@@ -549,6 +577,8 @@ class SimpleTopologyConfig(Config):
             domain.set_property("id", domain_name)
             topology.add_to_array("domains", domain)
 
+            has_go_ep = self.get((domain_name,"go-endpoint")) != None
+
             user = User()
             user.set_property("id", getpass.getuser())
             user.set_property("password_hash", "!")
@@ -638,6 +668,20 @@ class SimpleTopologyConfig(Config):
                     
                 domain.add_node(server_node)
 
+            for i in range(self.get((domain_name,"barebones-nodes"))):
+                bb_name = "%s-blank-%i" % (domain_name, i+1)
+
+                bb_node = Node()
+                bb_node.set_property("id", bb_name)
+                if self.get((domain_name,"nfs-nis")):  
+                    bb_node.set_property("depends", "node:%s" % server_name)
+                    bb_node.add_to_array("run_list", "role[domain-nfsnis-client]")
+                else:
+                    bb_node.add_to_array("run_list", "recipe[provision::gp_node]")  
+                    bb_node.add_to_array("run_list", "recipe[provision::domain_users]")              
+                               
+                domain.add_node(bb_node)
+
             if self.get((domain_name,"login")):            
                 login_node = Node()
                 login_node.set_property("id", "%s-login" % domain_name)
@@ -645,6 +689,7 @@ class SimpleTopologyConfig(Config):
                     login_node.set_property("depends", "node:%s" % server_name)
                     login_node.add_to_array("run_list", "role[domain-nfsnis-client]")
                 else:
+                    login_node.add_to_array("run_list", "recipe[provision::gp_node]")
                     login_node.add_to_array("run_list", "recipe[provision::domain_users]")
                 login_node.add_to_array("run_list", "role[globus]")
                 domain.add_node(login_node)                
@@ -656,6 +701,7 @@ class SimpleTopologyConfig(Config):
                     myproxy_node.set_property("depends", "node:%s" % server_name)
                     myproxy_node.add_to_array("run_list", "role[domain-nfsnis-client]")
                 else:
+                    myproxy_node.add_to_array("run_list", "recipe[provision::gp_node]")
                     myproxy_node.add_to_array("run_list", "recipe[provision::domain_users]")
                 myproxy_node.add_to_array("run_list", "role[domain-myproxy]")
                 domain.add_node(myproxy_node)
@@ -667,10 +713,17 @@ class SimpleTopologyConfig(Config):
                     gridftp_node.set_property("depends", "node:%s" % server_name)
                     gridftp_node.add_to_array("run_list", "role[domain-nfsnis-client]")
                 else:
+                    gridftp_node.add_to_array("run_list", "recipe[provision::gp_node]")
                     gridftp_node.add_to_array("run_list", "recipe[provision::domain_users]")     
-                if self.get((domain_name,"go-endpoint")) != None:
-                    gridftp_node.add_to_array("run_list", "recipe[globus::go_cert]")
-                gridftp_node.add_to_array("run_list", "role[domain-gridftp]")
+                    
+                if has_go_ep:
+                    if self.get((domain_name,"go-gc")):
+                        gridftp_node.add_to_array("run_list", "role[domain-gridftp-gc]")
+                    else:
+                        gridftp_node.add_to_array("run_list", "recipe[globus::go_cert]")
+                        gridftp_node.add_to_array("run_list", "role[domain-gridftp-default]")
+                else:                
+                    gridftp_node.add_to_array("run_list", "role[domain-gridftp-default]")
                 domain.add_node(gridftp_node)                
             
             if self.get((domain_name,"galaxy")):
@@ -681,6 +734,7 @@ class SimpleTopologyConfig(Config):
                     galaxy_node.set_property("depends", "node:%s" % server_name)
                     galaxy_node.add_to_array("run_list", "role[domain-nfsnis-client]")
                 else:
+                    galaxy_node.add_to_array("run_list", "recipe[provision::gp_node]")
                     galaxy_node.add_to_array("run_list", "recipe[provision::domain_users]")     
                     galaxy_node.add_to_array("run_list", "recipe[galaxy::galaxy-globus-common]")     
 
@@ -708,6 +762,7 @@ class SimpleTopologyConfig(Config):
                     lrm_node.set_property("depends", "node:%s" % server_name)
                     lrm_node.add_to_array("run_list", "role[domain-nfsnis-client]")
                 else:
+                    lrm_node.add_to_array("run_list", "recipe[provision::gp_node]")
                     lrm_node.add_to_array("run_list", "recipe[provision::domain_users]")                    
                 lrm_node.add_to_array("run_list", role)
                 domain.add_node(lrm_node)
@@ -722,23 +777,27 @@ class SimpleTopologyConfig(Config):
                     if self.get((domain_name,"nfs-nis")):
                         wn_node.add_to_array("run_list", "role[domain-nfsnis-client]")          
                     else:
+                        wn_node.add_to_array("run_list", "recipe[provision::gp_node]")
                         wn_node.add_to_array("run_list", "recipe[provision::domain_users]")                                  
                     wn_node.add_to_array("run_list", workernode_role)
                     domain.add_node(wn_node)
 
                     clusternode_host += 1
             
-            if self.get((domain_name,"go-endpoint")) != None:
+            if has_go_ep:
                 goep = GOEndpoint()
                 gouser, goname = self.get((domain_name,"go-endpoint")).split("#")
                 goep.set_property("user", gouser)
                 goep.set_property("name", goname)
+                goep.set_property("public", False)
                 goep.set_property("gridftp", "node:%s-gridftp" % domain_name)
                 
                 if self.get((domain_name,"go-auth")) == "myproxy":
                     goep.set_property("myproxy", "node:%s-myproxy" % domain_name)
                 else:
                     goep.set_property("myproxy", "myproxy.globusonline.org")
+
+                goep.set_property("globus_connect_cert", self.get((domain_name,"go-gc")))
                     
                 domain.add_to_array("go_endpoints", goep)
                 
